''**********************************************
''*  NTSC 160x160x16 color Video Driver v0.1   *
''*         (C) 2009 Hitt Consulting           *
''*         terry@hittconsulting.com           *
''**********************************************

' NOTE: Works at 80MHz clock ONLY !!!

' This video driver generates a bitmapped display of 160 horizonal by 192 vertical pixels
' Each pixel may be one of 16 colors.

' The pixels are not output in the normal way (using the propeller colorburst).
' Instead they are output directly at 4x the colorburst frequency.
' This causes the TV to intprete them as a phase change. For example if the four pixel bits are 0011 that is exactly
'  in phase with the colorburst, and 1100 would be 180 out of phase with the colorburst.

' Vertical layout: VSync 4, Top Margin 49, Video 160, Bottom Margin 49 = 262 lines frame
'  Top and bottom margins can be adjusted, as long as the total remains 262 lines per frame   

' Horizonal layout: Sync 17(68), Porch 4(16), Colorburst 10(40), Porch 4(16), Left Margin 12, Video 160, Right Margin 21 = 228 colorburst cycle times
'  Changed to Sysnc 16, Porch 4, Colorburst 12, Porch 4, Left margin 12, Video 160, Right Margin 16 = 224 colorburst cycle times

' COLORS:
' 0 = BLACK
' 1 = DARK MAGENTA
' 2 = BLUE
' 3 = LIGHT PURPLE
' 4 = DARK GREEN
' 5 = GRAY1
' 6 = CYAN
' 7 = LIGHT BLUE
' 8 = BROWN
' 9 = RED
' 10 = GRAY2
' 11 = LIGHT MAGENTA
' 12 = GREEN
' 13 = ORANGE/YELLOW
' 14 = LIGHT GREEN
' 15 = WHITE

' FOR PROPELLER DEMOBOARD

 
CON
' Set up the processor clock in the standard way for 80MHz
  _XINFREQ = 5_000_000 + 0000                           ' Demoboard
  _CLKMODE = XTAL1 + PLL16X                             ' Demoboard                                                     
   
 
VAR
  LONG cog           ' Video cog number (used to stop cog)

    
PUB Start(VideoMemoryAddr)
  ' Start video COG
  cog:=CogNew(@Color16Asm, VideoMemoryAddr)


PUB Stop
  if cog
    cogstop(cog~ - 1)


DAT
              ORG 0
Color16Asm    MOV _memStart, par     ' Get start of video memory
              MOVI CTRA,#%000001_111 ' Video PLL x16
              MOV VCFG, VideoVCFG    ' VGA
              MOV FRQA, VideoFRQA    ' 5.369318MHz 
              MOV DIRA, VideoDIRA    ' Setup output pins
              MOV VSCL, VideoVSCL    ' 6 clocks per pixel; 32 pixels per frame 

' NOTE: CHECK INTO WAITVID HANGING IF NOT CONFIGURED IN THE PROPER ORDER


' Generate active video lines
NewFrame
' Generate VSync
              MOV _vidVCntr,#4         ' 4 Lines of Sync for VSync
VSync2
              MOV _vidHCntr,#28
VSync1
              WAITVID VideoSync,#0
              DJNZ _vidHCntr,#VSync1
              DJNZ _vidVCntr,#VSync2              
' Generate Top blank lines
              MOV _vidVCntr,#49
'   Generate HSync
TopBlank
              WAITVID VideoSync,#0 
              WAITVID VideoSync,#0
              WAITVID VideoCBScreen,ColorBurst1
              WAITVID VideoCBScreen,ColorBurst2
              WAITVID VideoCBScreen,ColorBurst3
              MOV _vidHCntr,#23
TopBlankLine
              WAITVID VideoScreen,#0
              DJNZ _vidHCntr,#TopBlankLine
              DJNZ _vidVCntr,#TopBlank
              
' Generate active video
              MOV _memAddr,_memStart   ' Reset current video address to start of video memory
              MOV _vidVCntr,#160           ' Prepare to generate 160 video lines
NextLine
'   Generate HSync
              WAITVID VideoSync,#0 
              WAITVID VideoSync,#0
'   Generate Left Margin
              WAITVID VideoCBScreen,ColorBurst1
              WAITVID VideoCBScreen,ColorBurst2
              WAITVID VideoCBScreen,ColorBurst3
              WAITVID VideoScreen,#0
              
'   Generate video line
              MOV _vidHCntr,#20        ' Each line is 640 pixels 640/32 = 20
NextSegment
              RDLONG _vidSegment,_memAddr   
              WAITVID VideoScreen,_vidSegment   
'             WAITVID VideoScreen,testSegment
              ADD _memAddr,_640
              DJNZ _vidHCntr,#NextSegment
              SUB _memAddr,_12796
'   Generate Right Margin
              WAITVID VideoScreen,#0
              WAITVID VideoScreen,#0

              DJNZ _vidVCntr,#NextLine

' Generate Bottom blank lines
              MOV _vidVCntr,#49
'   Generate HSync
BotBlank
              WAITVID VideoSync,#0 
              WAITVID VideoSync,#0
              WAITVID VideoCBScreen,ColorBurst1
              WAITVID VideoCBScreen,ColorBurst2
              WAITVID VideoCBScreen,ColorBurst3
              MOV _vidHCntr,#23
BotBlankLine
              WAITVID VideoScreen,#0
              DJNZ _vidHCntr,#BotBlankLine
              DJNZ _vidVCntr,#BotBlank

              JMP #NewFrame
              
' Constants for Color16Asm
_640                     LONG 640
_12796                   LONG 12796

VideoVCFG                LONG %0_01_0_0_0_000_00000000000_001_0_11110000
VideoFRQA                LONG $112E_8B9E   ' = 5.369318Mhz FROM 80Mhz clock x 16PLL = 85.909089MHz (24xColorburst)
VideoVSCL                LONG $06_0C0      ' Pixel clock = 24xColorburst / 6 = 4xColorburst pixel rate; 192(C0h) frame rate (6*32)
VideoDIRA                LONG %00000000_00000000_11110000_00000000 ' demoboard

VideoSync                LONG %0000_0000_0000_0000
VideoCBScreen            LONG %0011_0000_0010_0000
VideoScreen              LONG %0111_0000_0010_0000

ColorBurst1              LONG %1100_1100_1100_1100_1010_1010_1010_1010
ColorBurst2              LONG %1100_1100_1100_1100_1100_1100_1100_1100
ColorBurst3              LONG %0000_0000_0000_0000_1010_1010_1010_1010


_memStart                RES 1 ' Holds address of video memory
_memAddr                 RES 1 ' Hold address of video memory currently being displayed
_vidVCntr                RES 1 ' Vertical video line counter
_vidHCntr                RES 1 ' Horizonal video LONG counter 
_vidSegment              RES 1 ' Dot pattern currently being displayed
  
  